<?php

namespace NoccyLabs\Cups\Job;

use NoccyLabs\Cups\Printer\Printer;

class PrintJob
{
 
    const OPT_ORIENT_PORTRAIT  = "portrait";
    const OPT_ORIENT_LANDSCAPE = "landscape";
    
    const OPT_MEDIA_A4 = "media=A4";
    const OPT_MEDIA_LETTER = "media=Letter";
    const OPT_MEDIA_UPPER_TRAY = "media=Upper";
    const OPT_MEDIA_LOWER_TRAY = "media=Lower";

    const DUPLEX_DEFAULT = -1;
    const DUPLEX_NONE = 0;
    const DUPLEX_LONG = 1;
    const DUPLEX_SHORT = 2;

    const OPT_DUPLEX_NONE = "sides=one-sided";
    const OPT_DUPLEX_LONG = "sides=two-sided-long-edge";
    const OPT_DUPLEX_SHORT = "sides=two-sided-short-edge";

    const OPT_ORDER_NORMAL = "outputorder=normal";
    const OPT_ORDER_REVERSE = "outputorder=reverse";

    const OPT_FIT_TO_PAGE = "fit-to-page";
    
    /** @var string */
    protected $filename;
    /** @var Printer */
    protected $printer;
    /** @var array */
    protected $options = [];

    public function __construct(?string $filename, ?Printer $printer)
    {
        $this->filename = $filename;
        $this->printer = $printer;
    }

    public function setPrinter(Printer $printer)
    {
        $this->printer = $printer;
    }

    public function getPrinter(): ?Printer
    {
        return $this->printer;
    }

    public function setDocument(string $filename)
    {
        $this->filename = $filename;
    }

    public function getDocument(): ?string
    {
        return $this->filename;
    }

    public function setOption(string $name, $value=true)
    {
        $this->options[$name] = $value;
    }

    public function clearOption(string $name)
    {
        unset($this->options[$name]);
    }

    public function getOption(string $name)
    {
        return array_key_exists($name, $this->options) ? $this->options[$name] : null;
    }

    public function setDuplexMode(int $mode)
    {
        switch ($mode) {
            case self::DUPLEX_DEFAULT:
                $this->clearOption("sides");
                break;
            case self::DUPLEX_NONE:
                $this->setOption("sides", "one-sided");
                break;
            case self::DUPLEX_LONG:
                $this->setOption("sides", "two-sided-long-edge");
                break;
            case self::DUPLEX_SHORT:
                $this->setOption("sides", "two-sided-short-edge");
                break;
            default:
                throw new \InvalidArgumentException("Invalid duplex mode");
        }
        return $this;
    }

    public function setPageRange(?string $range)
    {
        if ($range) {
            $this->setOption("page-ranges", $range);
        } else {
            $this->clearOption("page-ranges");
        }
        return $this;
    }

    public function setMedia(?string $media)
    {
        if ($media === null) {
            $this->clearOption("media");
        } else {
            $this->setOption("media", $media);
        }
        return $this;
    }

    public function setNumberUp(int $pages)
    {
        $this->setOption("number-up", $pages);
        return $this;
    }

    public function getNumberUp(): int
    {
        return $this->getOption("number-up");
    }

    public function submit(bool $deleteAfterPrint=false)
    {

        $cmdl = [];

        $cmdl[] = "lpr";

        array_push($cmdl, escapeshellarg('-P'), escapeshellarg($this->printer->getName()));

        foreach ($this->options as $opt=>$val) {
            if ($val === true) {
                $cmdl[] = "-o";
                $cmdl[] = escapeshellcmd($opt);
            } else {
                $cmdl[] = "-o";
                $cmdl[] = escapeshellcmd($opt)."=".escapeshellarg($val);
            }
        }

        if ($deleteAfterPrint) {
            $cmdl[] = '-r';
        }

        $cmdl[] = escapeshellarg($this->filename);

        $cmdl = join(" ",$cmdl);

        printf("Submitting to %s\n", $this->printer);

        echo $cmdl."\n";
        exec($cmdl, $output, $ret);
        if ($ret != 0) {
            throw new \RuntimeException("Call to lpr failed");
        }
    }

}