<?php

namespace NoccyLabs\Cups\Printer;

use DateTime;
use InvalidArgumentException;

class Printer
{

    const PRINTER_STATE_IDLE = 3;
    const PRINTER_STATE_STOPPED = 5;

    private $name;

    private $alias;

    private $props = [];

    private $config = [];

    /**
     * Printer constructor. Can be used with either a printer name (or alias) or the
     * direct printcap line from the /etc/printcap
     * 
     * @param string 
     */
    public function __construct($printer)
    {
        if (strpos($printer, "|") === false) {

            $printerInfo = self::getPrinterInfo($printer);
            if (!$printerInfo) {
                throw new InvalidArgumentException("The Printer constructor expects a printcap line or a printer name or alias");
            }
            $printer = $printerInfo;
        }

        [$this->name, $info] = explode("|", $printer, 2);
        
        $info = explode(":", $info);
        $this->alias = array_shift($info);
        
        foreach ($info as $propstr) {
            if (strpos($propstr, "=") !== false) {
                [$prop,$val] = explode("=", $propstr, 2);
                $this->props[$prop] = $val;
            }
        }

        exec(escapeshellcmd("lpoptions") . " -p ". escapeshellarg($this->name), $output, $ret);
        if ($ret != 0) {
            print_r($output);
            return;
        }

        $printerOptions = array_shift($output);
        if (preg_match_all("/[^\s']+(?:'[^']*'[^\s']*)*|(?:'[^']*'[^\s']*)+/", $printerOptions, $match)) {
            foreach ($match[0] as $opts) {
                if (strpos($opts,"=") === false) continue;
                [$key,$val] = explode("=", $opts,2);
                if (strlen($val) > 2 && substr($val, 0, 1) == substr($val, -1, 1) && $val[0] == "'") {
                    $val = substr($val, 1, -1);
                }
                $this->config[$key] = $val;
            }
        }

    }

    /**
     * @static
     * 
     * Get information on a specific printer by its name or alias.
     * 
     * @return array
     */
    public static function getPrinterInfo(string $nameOrAlias): ?string
    {
        $printers = PrinterIterator::getPrintcapList();
        foreach ($printers as $printerinfo) {
            if (preg_match("/^({$nameOrAlias})\|/i", $printerinfo, $match)) {
                return $printerinfo;
            }
            if (preg_match("/\|({$nameOrAlias}):/i", $printerinfo, $match)) {
                return $printerinfo;
            }
        }
        return null;
    }

    /**
     * Return the cups devicename of the printer.
     * 
     * @return string
     */
    public function getName(): string
    {
        return $this->name;
    }

    /**
     * Return the valid alias for the printer.
     * 
     * @return string
     */
    public function getAlias(): ?string
    {
        return $this->alias;
    }

    /**
     * Return the make and model of the printer.
     * 
     * @return string
     */
    public function getMakeAndModel(): ?string
    {
        return $this->getConfigKey("printer-make-and-model");
    }

    /**
     * Return the numeric state of this printer, to match against the PRINTER_STATE_*
     * constants.
     * 
     * @return int|null
     */
    public function getState(): ?int
    {
        return $this->getConfigKey("printer-state");
    }

    /**
     * Get the reason this printer is unavailable
     * 
     * @return string|null
     */
    public function getStateReason(): ?string
    {
        return $this->getConfigKey("printer-state-reasons");
    }

    /**
     * Get the DateTime when the printer status was last updated.
     * 
     * @return \DateTime
     */
    public function getStateUpdated(): DateTime
    {
        $dt = new DateTime();
        $changed = $this->getConfigKey("printer-state-change-time");
        if ($changed) {
            $dt->setTimestamp($changed);
        }
        return $dt;
    }

    public function getProps(): array
    {
        return $this->props;
    }

    public function getPropKey(string $key, $defaultValue=null)
    {
        return array_key_exists($key, $this->props) ? $this->props[$key] : $defaultValue;
    }

    public function getConfig(): array
    {
        return $this->config;
    }

    public function getConfigKey(string $key, $defaultValue=null)
    {
        return array_key_exists($key, $this->config) ? $this->config[$key] : $defaultValue;
    }

    public function asJson()
    {
        return json_encode([
            'name' => $this->name,
            'alias' => $this->alias,
            'props' => $this->props,
            'config' => $this->config,
        ], JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
    }

    public function __toString()
    {
        $makeModel = $this->getConfigKey("printer-make-and-model", "?");
        $printerInfo = $this->getConfigKey("printer-info", "?");
        $deviceUri = $this->getConfigkey('device-uri', "?");

        return sprintf("%s (%s) on %s", $printerInfo, $makeModel, $deviceUri);
    }
}
